-----------------------------------------
-- Initialize Addon default variables ---
-----------------------------------------
	-- Set DefaultVariables
	DefaultName = ""
	DefaultActive = true
	DefaultWelcomeMessages = true

	-- Create frame and register events 
	local frame = CreateFrame("FRAME"); -- Need a frame to respond to events
	frame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
	frame:RegisterEvent("PLAYER_LOGIN"); -- Fired when logged in
	frame:RegisterEvent("PLAYER_LOGOUT"); -- Fired when about to log out
	frame:RegisterEvent("QUEST_TURNED_IN"); -- Register quest turn in event

-------------------------------------------
-- Initialize and handles SavedVariables --
-------------------------------------------
	function frame:OnEvent(event, arg1)
 		if event == "ADDON_LOADED" and arg1 == "DisplayName" then
			-- Check if addon is never used before.
			if DisplayNameActive == nil then
				DisplayNameActive = true;			-- Make addon active
				CurrentName = UnitName("player");	-- Use character name
				ShowWelcomeMessage = true;			-- Enable welcome message
				CustomDisplayName = "";				-- Set DisplayName variable to empty. It's improtant to turn it into something than nil.

			-- Set variables if DisplayNameActive is true
			elseif DisplayNameActive == true then
				if CustomDisplayName == "" then
					CurrentName = UnitName("player");
				else
					CurrentName = CustomDisplayName
				end
				if ShowWelcomeMessage == nil then
					ShowWelcomeMessage = true
				end
				else
					CurrentName = UnitName("player");
				end

		-- Execute this line upon player login.
		elseif event == "PLAYER_LOGIN" then
			if ShowWelcomeMessage == true and DisplayNameActive == true then
				local WMSG = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r DisplayName (v1.0.0)"
				local WHELP = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r /dnhelp to see commands."
				print(WMSG);
				print(WHELP);
			end
		end
	end

	frame:SetScript("OnEvent", frame.OnEvent);

-------------------------------------
-- Handles the display name change --
-------------------------------------
	-- Reload UI function
	function Reload(self)
		ReloadUI();
	end

	-- Change display name
	function SetCustomName(unit, fontString)
		if UnitIsUnit(unit, "player") then
			fontString:SetText(CurrentName);
		end
	end

	-- Update player portrait name, UI needs to be reloaded.
	hooksecurefunc("UnitFrame_Update", function(frame, isParty)
		if frame.name then
			SetCustomName(frame.overrideName or frame.unit, frame.name)
		end
	end)

	-- Update target portrait name
	hooksecurefunc("CompactUnitFrame_UpdateName", function(frame)
		SetCustomName(frame.unit, frame.name)
	end)

-------------------------------
-- Stores all slash commands --
-------------------------------
 	SLASH_HELP1 = "/dnhelp" -- Show popup window for creating new name.
 	SLASH_SETNAME1 = "/dnsetname" -- Show popup window for creating new name.
 	SLASH_ENABLEWMSG1 = "/dnwe" -- Show startup welcome message.
 	SLASH_DISABLEWMSG1 = "/dnwd" -- Disable startup welcome message.
 	SLASH_ACTIVE1 = "/dnactive" -- Activate custom name.
 	SLASH_INACTIVE1 = "/dninactive" -- Deactivate custom name.
 	SLASH_ACTIVECHECK1 = "/dncheck" -- check if addon is active or not and your displayname.
 	SLASH_RESET1 = "/dnreset" -- Reset Display Name to default.

-------------------------------------------
-- Run Slash Commands when typed in chat --
-------------------------------------------
	-- Run /dnsetname command
	-- Opens popup window where player can type new display name.
	function SlashCmdList.SETNAME()
		StaticPopup_Show("DisplayNameChange")
	end
	-- Run /dnwe command
	-- Enables welcome message
	function SlashCmdList.ENABLEWMSG()
		ShowWelcomeMessage = true;
	end
	-- Run /dnwd command
	-- Disables welcome message
	function SlashCmdList.DISABLEWMSG()
		ShowWelcomeMessage = false;
	end
	-- Run /dnreset command
	-- Resets display NAme back to defaults
	function SlashCmdList.RESET()
		DisplayNameActive = DefaultActive;
		CustomDisplayName = DefaultName;
		ShowWelcomeMessage = DefaultWelcomeMessages;
		Reload();
	end
	-- Run /dncheck command
	-- Check if addon is active or not.
	function SlashCmdList.ACTIVECHECK()
		local Active = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r DisplayName is active."
		local Inactive = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r DisplayName is inactive."
		local YourName = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r Your name: " .. CustomDisplayName .. ""
		local WelcomeMsgTrue = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r Welcome Message: Enabled"
		local WelcomeMsgFalse = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r Welcome Message: Disabled"
		if DisplayNameActive == true then
			print(Active)
		else
			print(Inactive)
		end
		print(YourName)
		if ShowWelcomeMessage == true then
			print(WelcomeMsgTrue)
		else
			print(WelcomeMsgFalse)
		end
	end
	-- Run /dnactivecommand
	-- Make custom display name active, using custom name instead
	function SlashCmdList.ACTIVE()
		local Message = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r is already active."
		if DisplayNameActive == true then
			print(Message)
		else
			DisplayNameActive = true;
			Reload();
		end
	end
	-- Run /dninactive command
	-- Make custom display name inactive, using character name instead
	function SlashCmdList.INACTIVE()
		local Message = "|cffffffff[|r|cffFF4500Display Name|r|cffffffff]|r|cffffffff|r is already inactive."
		if DisplayNameActive == false then
			print(Message)
		else
			DisplayNameActive = false;
			Reload();
		end
	end
	-- Run /dnhelp command
	-- Shows a list of commands that can be used
	function SlashCmdList.HELP(msg)
		local Help = "|cffffffff[|r|cffFF4500Display Name Help|r|cffffffff]|r|cffffffff|r"
		print(Help);
		print("/dnsetname - Set custom name")
		print("/dnactive - Use Display Name")
		print("/dninactive - Use Character Name")
		print("/dnwe - Enable welcome message")
		print("/dnwd - Disable welcome message")
		print("/dncheck - Check is active or inactive")
	end

-------------------------------------------
-- Show popup window to type in new name --
-------------------------------------------
	local NewName
		StaticPopupDialogs.DisplayNameChange = {
			OnShow = function(self)
				self.editBox:SetText(CustomDisplayName);
			end,
			text = "Display Name",
			button1 = "Accept",
			button2 = "Cancel",
			OnUpdate = function(self)
				local Cache = self.editBox:GetText()
				if Cache == "" then
					self.button1:SetEnabled(false);
				elseif Cache == CurrentName then
					self.button1:SetEnabled(false);
				else
					self.button1:SetEnabled(true);
				end
			end,
			OnAccept = function(self)
				NewName = self.editBox:GetText()
				CustomDisplayName = NewName
				CurrentName = CustomDisplayName
				Reload();
			end,
			hasEditBox = 1,
			OnCancel = function(self)
				StaticPopup_Hide("DisplayNameChange")
			end,
		}